/*
 * This source code (including its associated software) is owned by
 * StorageCraft Technology Corporation and is protected by United States and
 * international intellectual property law, including copyright laws, patent
 * laws, and treaty provisions. Use of the source code without modification in
 * connection with StorageCraft products is permitted subject to the End-User
 * License Agreement for StorageCraft Products ("EULA"). A copy of the
 * StorageCraft EULA is available here:
 * https://www.storagecraft.com/legal/license-agreement. Any other use,
 * reproduction, or disclosure without the written permission of StorageCraft
 * Technology Corporation is prohibited.
 *
 * © 2014-2019 StorageCraft Technology Corporation. All rights reserved.
 */

#include <linux/slab.h>

#include "cow_range.h"

#define PTRS_PER_RANGE 32  /* must be power of 2 */

struct cow_range {
	struct rb_node node;
	u32 start;
	struct cow_buffer *cows[PTRS_PER_RANGE];
};

struct cow_buffer *cow_search_range(struct rb_root *root, u32 index)
{
	struct rb_node *node = root->rb_node;

	unsigned int offset = index & (PTRS_PER_RANGE-1);
	u32 start = index - offset;

	while (node) {
		struct cow_range *range = container_of(node, struct cow_range, node);

		if (start < range->start)
			node = node->rb_left;
		else if (start >= range->start + PTRS_PER_RANGE)
			node = node->rb_right;
		else
			return range->cows[offset];
	}
	return NULL;
}

int cow_add_range(struct rb_root *root, u32 index, struct cow_buffer *cow)
{
	struct rb_node **new = &(root->rb_node), *parent = NULL;
	struct cow_range *range;

	unsigned int offset = index & (PTRS_PER_RANGE-1);
	u32 start = index - offset;

	while (*new) {
		range = container_of(*new, struct cow_range, node);

		parent = *new;
		if (index < range->start) {
			new = &((*new)->rb_left);
		} else if (index >= range->start + PTRS_PER_RANGE) {
			new = &((*new)->rb_right);
		} else {
			range->cows[offset] = cow;
			return 0;
		}
	}

	/* Add new node and rebalance tree. */
	range = kzalloc(sizeof(struct cow_range), GFP_KERNEL);
	if (!range)
		return -ENOMEM;

	RB_CLEAR_NODE(&range->node);
	range->start = start;
	rb_link_node(&range->node, parent, new);
	rb_insert_color(&range->node, root);

	return 0;
}
