/*
 * This source code (including its associated software) is owned by
 * StorageCraft Technology Corporation and is protected by United States and
 * international intellectual property law, including copyright laws, patent
 * laws, and treaty provisions. Use of the source code without modification in
 * connection with StorageCraft products is permitted subject to the End-User
 * License Agreement for StorageCraft Products ("EULA"). A copy of the
 * StorageCraft EULA is available here:
 * https://www.storagecraft.com/legal/license-agreement. Any other use,
 * reproduction, or disclosure without the written permission of StorageCraft
 * Technology Corporation is prohibited.
 *
 * © 2014-2019 StorageCraft Technology Corporation. All rights reserved.
 */

#include <linux/module.h>

#include "debug.h"
#include "export.h"
#include "filter.h"
#include "stcvsm_version.h"
#include "table.h"
#include "vdiff.h"

int deny_export;

void *stcvsm_export_vdiff(unsigned int cmd)
{
	int i;
	int vdiff_idx;
	int num_vdiffs;
	struct exported_state *state;
	struct block_device_filter *table = stcvsm_filter_table();

	switch (cmd) {
	case 0:  /* start transaction */
		stcvsm_info(LOG_MODULE, "start export transaction\n");
		if (deny_export)
			return ERR_PTR(-EACCES);
		if (quiesce_filters(1)) {
			quiesce_filters(0);
			return ERR_PTR(-EBUSY);
		}

		num_vdiffs = 0;
		for (i = 0; i < STCVSM_MAX_FILTERS; ++i)
			num_vdiffs += table[i].num_vdiffs;

		state = kmalloc(sizeof(struct exported_state) +
				sizeof(struct exported_filter) * num_vdiffs, GFP_KERNEL);
		if (state == NULL) {
			quiesce_filters(0);
			return ERR_PTR(-ENOMEM);
		}
		state->stcvsm_version = (STCVSM_MAJOR << 24) | (STCVSM_MINOR << 16) |
			(STCVSM_PATCH << 8) | (STCVSM_BUILD & 0xff);
		state->num_vdiffs = num_vdiffs;
		state->reserved1 = 0;
		state->reserved2 = 0;
		state->stcvsm_ioctl_vdiff = stcvsm_ioctl_vdiff;

		num_vdiffs = 0;
		for (i = 0; i < STCVSM_MAX_FILTERS; ++i) {
			for (vdiff_idx = 0; vdiff_idx < table[i].num_vdiffs; ++vdiff_idx) {
				struct vdiff_context *vdiff = table[i].vdiffs[vdiff_idx];

				state->filters[num_vdiffs].major = MAJOR(vdiff->vd_dev);
				state->filters[num_vdiffs].minor = MINOR(vdiff->vd_dev);
				state->filters[num_vdiffs].orig_reqfn = table[i].bdf_orig_reqfn;
				num_vdiffs++;
			}
		}
		state->filters[num_vdiffs].major = 0;
		state->filters[num_vdiffs].minor = 0;
		state->filters[num_vdiffs].orig_reqfn = NULL;

		return state;
	case 1:  /* commit transaction */
		stcvsm_info(LOG_MODULE, "commit export transaction\n");
		for (i = 0; i < STCVSM_MAX_FILTERS; ++i)
			table[i].bdf_orig_reqfn = NULL;
		shutdown_filters();
		return NULL;
	case 2:  /* cancel transaction */
		stcvsm_info(LOG_MODULE, "cancel export transaction\n");
		quiesce_filters(0);
		return NULL;
	default:
		return ERR_PTR(-EINVAL);
	}
}
