/*
 * This source code (including its associated software) is owned by
 * StorageCraft Technology Corporation and is protected by United States and
 * international intellectual property law, including copyright laws, patent
 * laws, and treaty provisions. Use of the source code without modification in
 * connection with StorageCraft products is permitted subject to the End-User
 * License Agreement for StorageCraft Products ("EULA"). A copy of the
 * StorageCraft EULA is available here:
 * https://www.storagecraft.com/legal/license-agreement. Any other use,
 * reproduction, or disclosure without the written permission of StorageCraft
 * Technology Corporation is prohibited.
 *
 * © 2014-2019 StorageCraft Technology Corporation. All rights reserved.
 */

#ifndef STCVSM_EXPORT_H
#define STCVSM_EXPORT_H

#include <linux/blkdev.h>

struct exported_filter {
	int major;
	int minor;
	make_request_fn *orig_reqfn;
};

struct exported_state {
	u32 stcvsm_version;  /* major minor patch build */
	u32 num_vdiffs;
	u32 reserved1;
	u32 reserved2;
	int (*stcvsm_ioctl_vdiff)(dev_t dev, unsigned int cmd, void *param);
	struct exported_filter filters[1];
};

/** Manages the sending end of the upgrade transaction, which is used to transfer incremental
 * tracking state and ownership to another driver.  While the transaction is open, the driver may
 * not create or destroy vdiff or snapshot instances.  If the transaction completes successfully,
 * the driver remains quiesced and should be unloaded.
 *
 * 0: start transaction; return is the exported state, or NULL on error
 * 1: commit transaction; driver has lost ownership of all make_request_fn
 * 2: cancel transaction
 */
void *stcvsm_export_vdiff(unsigned int cmd);

#endif
