/*
 * This source code (including its associated software) is owned by
 * StorageCraft Technology Corporation and is protected by United States and
 * international intellectual property law, including copyright laws, patent
 * laws, and treaty provisions. Use of the source code without modification in
 * connection with StorageCraft products is permitted subject to the End-User
 * License Agreement for StorageCraft Products ("EULA"). A copy of the
 * StorageCraft EULA is available here:
 * https://www.storagecraft.com/legal/license-agreement. Any other use,
 * reproduction, or disclosure without the written permission of StorageCraft
 * Technology Corporation is prohibited.
 *
 * © 2014-2019 StorageCraft Technology Corporation. All rights reserved.
 */

/*
 * Utility package - UASSERT macros
 * 64bit ready
 */
#ifndef _UASSERT_H
#define _UASSERT_H

#ifdef DEBUG

#include "stcvsm_os.h"

#define UASSERT(Cond, Msg)	\
	do { if (!(Cond)) {				\
		printk(STCVSM_EMERG "ASSERTION FAILED %s %s()-%d: '%s'!\n",	\
		__FILE__, __func__, __LINE__, (Msg));		\
		BUG();					\
		} } while (0)

#define UASSERTD(Cond, Msg, Val) \
	do { if (!(Cond)) {				\
		printk(STCVSM_EMERG "ASSERTION FAILED %s %s()-%d: '%s' [%d]!\n",	\
		__FILE__, __func__, __LINE__, (Msg), (Val));	\
		BUG();					\
		} } while (0)

#define UASSERTP(Cond, Msg, Ptr) \
	do { if (!(Cond)) {				\
		printk(STCVSM_EMERG "ASSERTION FAILED %s %s()-%d: '%s' [%p]!\n",	\
		__FILE__, __func__, __LINE__, (Msg), (Ptr));	\
		BUG();					\
		} } while (0)
#else

#define UASSERT(Cond, Msg)            do {} while (0)
#define UASSERTD(Cond, Msg, Val)      do {} while (0)
#define UASSERTP(Cond, Msg, Ptr)      do {} while (0)

#endif	/* DEBUG */

#endif	/* _UASSERT_H */
