/*
 * This source code (including its associated software) is owned by
 * StorageCraft Technology Corporation and is protected by United States and
 * international intellectual property law, including copyright laws, patent
 * laws, and treaty provisions. Use of the source code without modification in
 * connection with StorageCraft products is permitted subject to the End-User
 * License Agreement for StorageCraft Products ("EULA"). A copy of the
 * StorageCraft EULA is available here:
 * https://www.storagecraft.com/legal/license-agreement. Any other use,
 * reproduction, or disclosure without the written permission of StorageCraft
 * Technology Corporation is prohibited.
 *
 * © 2014-2019 StorageCraft Technology Corporation. All rights reserved.
 */

#include <linux/init.h>
#include <linux/module.h>

#include "debug.h"
#include "export.h"
#include "filter.h"
#include "import.h"
#include "stcvsm_version.h"

MODULE_LICENSE("Copyright by StorageCraft Technology Corporation");
MODULE_VERSION(STCVSM_VERSION);

static unsigned long export_addr;
module_param(export_addr, ulong, 0);
MODULE_PARM_DESC(export_addr, "Address of stcvsm_export_vdiff, for dynamic driver update");

int orig_write_filter(void) { return 0; }
void cow_bdev_write_endio(void) {}
void *get_snapshot_bio(void) { return NULL; }
void put_snapshot(void) {}
void stcvsm_init_proc_filter(void) {}
void stcvsm_remove_proc_filter(void) {}

static int __init stcvsm_upgrade_init(void)
{
	int err = 0;
	void *(*stcvsm_export_vdiff)(unsigned int cmd);
	if (export_addr == 0)
		return -EINVAL;
	stcvsm_export_vdiff = (void *(*)(unsigned int cmd))export_addr;

	init_filters();

	err = stcvsm_import_vdiff(stcvsm_export_vdiff);

	return err;
}

static void __exit stcvsm_upgrade_exit(void)
{
	/* VDiff is normally torn down when the upgrade driver's state is exported.  Clean up in
	 * case the export was canceled.
	 *
	 * Do not attempt to un-quiesce the old driver if the upgrade driver's state has not been
	 * exported; the old driver may have been unloaded.
	 */
	shutdown_filters();
}

module_init(stcvsm_upgrade_init);
module_exit(stcvsm_upgrade_exit);
