/*
 * This source code (including its associated software) is owned by
 * StorageCraft Technology Corporation and is protected by United States and
 * international intellectual property law, including copyright laws, patent
 * laws, and treaty provisions. Use of the source code without modification in
 * connection with StorageCraft products is permitted subject to the End-User
 * License Agreement for StorageCraft Products ("EULA"). A copy of the
 * StorageCraft EULA is available here:
 * https://www.storagecraft.com/legal/license-agreement. Any other use,
 * reproduction, or disclosure without the written permission of StorageCraft
 * Technology Corporation is prohibited.
 *
 * © 2014-2019 StorageCraft Technology Corporation. All rights reserved.
 */

#ifndef STCVSM_WORKQUEUE_H
#define STCVSM_WORKQUEUE_H

#include <linux/spinlock.h>
#include <linux/list.h>
#include <linux/wait.h>

/** Simple non-GPL workqueue.
 *
 * Linux's workqueue implementation is only available to GPL code (unless you only use the system's
 * generic workqueues).  This is a simpler-but-sufficient implementation (not per-CPU, no
 * CPU-binding, no delayed work, etc).
 */
struct wq {
	spinlock_t lock;
	struct list_head work;
	wait_queue_head_t more_work;
	struct task_struct *thread;
	const char *name;
	void (*cleanup_fn)(void *data);
	void *cleanup_data;
};

struct wq_work;
typedef void (*wq_work_fn)(struct wq_work *);
struct wq_work {
	struct list_head list;
	wq_work_fn fn;
	long data;
};

/** Initializes the workqueue.
 */
void wq_init(struct wq *wq, const char *name, ...);

/** Starts the workqueue; must balance with wq_stop.
 */
int wq_start(struct wq *wq);

/** Runs the work.
 */
void wq_insert_work(struct wq *wq, struct wq_work *work);

/** Finishes queued work and then stops the workqueue.  Safe to call from within the workqueue.
 */
int wq_stop(struct wq *wq, void (*cb)(void *data), void *data);

void wq_deinit(struct wq *wq);

#endif
